import random

import requests
import sqlobject

from ..base import CommandBridle
from ..base import BridleModel


class SexyPaddyURL(BridleModel):
    url = sqlobject.StringCol()
    added_by = sqlobject.ForeignKey('User')
    invoked = sqlobject.IntCol(default=0)


class SexyPaddy(CommandBridle):

    class Meta(CommandBridle.Meta):
        command = 'sexypaddy'
        display_name = "Sexy Paddy"
        display_icon = (
            "https://secure.gravatar.com/avatar/"
            "6f836d78a34bd1e63b51e9a700188e5c.jpg"
        )
        description = 'So you want some sexy paddy, huh?'
        help_text = [
            'Usage:',
            '\t`/horse sexypaddy` - get a random paddy',
            '\t`/horse sexypaddy list` - get a list of paddies'
            '\t`/horse sexypaddy add <url>` - add a new paddy'
        ]

    def execute(self, user, channel, operands):
        print operands
        if len(operands) == 0:
            return self.random_paddy(user, channel)
        elif operands[0] == 'add':
            url = operands[1]
            response = requests.get(url)
            if response.status_code != 200:
                return "Can't seem to access URL. Not added.".format(url)
            else:
                return self.add_paddy(user, url)
        elif operands[0] == 'remove':
            url = operands[1]
            return self.remove_paddy(url)
        elif operands[0] == 'list':
            return self.list_paddies()

    def random_paddy(self, user, channel):
        paddies = list(SexyPaddyURL.select())
        if len(paddies) == 0:
            return "No paddies found, try using `/horse sexypaddy add <url>`."
        selected = random.choice(paddies)
        selected.set(invoked=selected.invoked + 1)
        self.message(channel, selected.url)

    def add_paddy(self, user, url):
        exists = SexyPaddyURL.selectBy(url=url).getOne(None)
        if exists is None:
            SexyPaddyURL(url=url, added_by=user)
            return "Added to sexypaddy".format(url)
        else:
            return "URL was already added by {0}".format(
                exists.added_by.username
            )

    def remove_paddy(self, url):
        exists = SexyPaddyURL.selectBy(url=url).getOne(None)
        if exists is None:
            return "Can't find this URL."
        else:
            exists.destroySelf()
            return "URL deleted."

    def list_paddies(self):
        paddies = SexyPaddyURL.select()
        return "\n".join([
            "{0} - {1}".format(p.added_by.username, p.url)
            for p in paddies
        ])



import random
import requests
from datetime import datetime
from cStringIO import StringIO

import matplotlib.pyplot as plt

import horse.config
import horse.utils.imgur
from horse.bridles.base import CommandBridle


class ForecastIO(CommandBridle):

    class Meta(CommandBridle.Meta):
        command = 'weather'
        description = "Pulls and represents weather info from forecast.io"
        help_text = ["Usage: `/horse weather [location]`"]

        display_name = "Weather Forecast"
        display_icon = "http://ryangrieve.com/labs/slack_icons/weather.png"

    def __init__(self, jockey):
        super(ForecastIO, self).__init__(jockey)
        self.locations = {
            "belfast": "54.58970503,-5.92369347",
            "london": "51.52908935,-0.08757537"
        }

    def get_current_weather(self, location):
        coords = self.locations[location]
        url = "https://api.forecast.io/forecast/{0}/{1}?units=si".format(
            horse.config.FORECASTIO_API_TOKEN,
            coords
        )
        response = requests.get(url)
        if response.status_code == 200:
            forecast = response.json()
            return forecast
        else:
            return None

    def determine_location(self, operands):
        location = None
        for place in self.locations:
            if place in operands:
                location = place

        if not location:
            location = 'belfast'

        return location

    def execute(self, user, channel, operands):
        location = self.determine_location(operands)
        weather = self.get_current_weather(location)
        msg = "*%s*: _%s_, %.0fdeg\n%s %s %s" % (
            location.capitalize(),
            weather['currently']['summary'],
            weather['currently']['temperature'],
            weather['minutely']['summary'],
            weather['hourly']['summary'],
            weather['daily']['summary']
        )
        self.message(channel, msg)


class WeatherGraph(ForecastIO):

    class Meta(ForecastIO.Meta):
        _secret = True
        command = '_'

    def execute(self, user, channel, operands):
        location = self.determine_location(operands)
        weather = self.get_current_weather(location)
        data = self.get_data(weather, operands)
        plot = self.draw_plot(data, location)
        url = horse.utils.imgur.post_from_file(plot)
        self.message(channel, url)

    def get_data(self, weather):
        return []

    def get_title(self, location):
        return ""

    def draw_plot(self, data, location):
        output = StringIO()
        with plt.xkcd():
            plt.clf()
            plt.plot(*data, c=random.choice("bgrcmykw"))
            plt.gcf().autofmt_xdate()
            plt.title(self.get_title(location))
            plt.savefig(output, format="png")
        output.seek(0)
        return output


class RainGraph(WeatherGraph):

    class Meta(WeatherGraph.Meta):
        command = 'rain'

    def get_title(self, location):
        return "Rain intensity for the next 24 hours in {0}".format(
            location.capitalize()
        )

    def get_data(self, weather, operands):
        return (
            [
                datetime.fromtimestamp(d['time'])
                for d in weather['hourly']['data'][:24]
            ],
            [
                d['precipIntensity']
                for d in weather['hourly']['data'][:24]
            ]
        )


class TemperatureGraph(WeatherGraph):

    class Meta(WeatherGraph.Meta):
        command = 'temperature'

    def get_title(self, location):
        return "Temperature (C) for next 24 hours in {0}".format(
            location.capitalize()
        )

    def get_data(self, weather, operands):
        return (
            [
                datetime.fromtimestamp(d['time'])
                for d in weather['hourly']['data'][:24]
            ],
            [
                d['temperature']
                for d in weather['hourly']['data'][:24]
            ]
        )



import random

import requests

from ... import config
from ...models import Channel
from ..base import CommandBridle


class Quote(CommandBridle):

    class Meta(CommandBridle.Meta):
        command = 'quote'
        description = "Fetches a random quote shared in #quotes"
        help_text = ["Usage: `/horse quote`"]
        display_name = "+rehabstudio Quotes"

    def __init__(self, jockey):
        super(Quote, self).__init__(jockey)
        self.quotes_channel = Channel.selectBy(name='quotes').getOne(None)

    def get_random_quote(self):
        quotes = self.get_all_quotes()
        if quotes:
            quotes = self.filter_quotes(quotes)
            print quotes
            return random.choice(quotes)
        else:
            return None

    def get_all_quotes(self):
        url = "https://slack.com/api/files.list"
        params = {
            "token": config.SLACK_API_TOKEN,
            "types": "images",
            "count": 1000
        }
        response = requests.get(url, params=params)
        if response.status_code == 200:
            quotes = response.json()
            return quotes['files']
        else:
            return None

    def filter_quotes(self, quotes):
        filtered = []
        for quote in quotes:
            if self.quotes_channel.string_id in quote['channels']:
                filtered.append(quote)
        return filtered

    def execute(self, user, channel, operands):
        if len(operands) == 0:
            quote = self.get_random_quote()
            response = quote['url']
            self.message(channel, response)
        elif operands[0] == 'all':
            quotes = self.get_all_quotes()
            quotes = self.filter_quotes(quotes)
            response = "\n".join([q['url'] for q in quotes])
            self.message(user, response)



from base64 import b64encode
from cStringIO import StringIO

import requests
from PIL import Image, ImageSequence, ImageDraw, ImageFont

from ... import config
from ..base import CommandBridle
from ...utils.images2gif import GifWriter


class GifReact(CommandBridle):

    class Meta(CommandBridle.Meta):
        command = 'gif'
        description = 'Get an animated reaction from giphy.com'
        help_text = ['Usage: `/horse gif <query>`']

    def search(self, operands):
        response = requests.get(
            'http://api.giphy.com/v1/gifs/search?q={0}&api_key={1}'.format(
                "+".join(operands),
                config.GIPHY_API_TOKEN
            )
        )

        results = response.json()
        return results

    def get_image(self, results, operands):
        return results['data'][0]['images']['fixed_height']['url']

    def execute(self, user, channel, operands):

        if len(operands) == 0:
            return 'Can\'t search for nothing, mate'

        results = self.search(operands)
        if len(results['data']) > 0:
            self.message(
                channel,
                self.get_image(results, operands),
                username=user.real_name if user.real_name else user.name,
                icon_url=user.image
            )
        else:
            return 'Sorry, found nothing :('


class MemeReact(GifReact):

    class Meta(CommandBridle.Meta):
        command = 'meme'
        description = 'Make a meme using images from giphy.com'
        help_text = ['Usage: `/horse meme <caption>`']

    def get_image(self, results, operands):
        orig = results['data'][0]['images']['fixed_height']['url']
        return self.make_meme(orig, " ".join(operands))

    def make_meme(self, original_image, caption, shrink=1):
        response = requests.get(original_image)
        pil_img = Image.open(StringIO(response.content))
        duration = pil_img.info['duration'] / 1000.0

        frames = []
        for frame in ImageSequence.Iterator(pil_img):
            frames.append(frame.copy())

        gf = GifWriter()
        images = gf.convertImagesToPIL(frames, False, 0)

        durations = []
        xys = []
        disposes = []
        for im in images:
            durations.append(duration)
            xys.append((0, 0))
            disposes.append(2)

            width, height = im.size
            if shrink != 1:
                im.thumbnail(
                    (width * shrink, height * shrink),
                    Image.ANTIALIAS
                )
                width, height = im.size
            draw = ImageDraw.Draw(im)

            text_width = 99999
            font_size = 33
            while text_width > width * 0.8:
                font_size -= 1
                font = ImageFont.truetype('assets/fonts/impact.ttf', font_size)
                text_width, text_height = draw.textsize(
                    caption.upper(),
                    font=font
                )

            # horrible outline hack
            draw.text(
                ((width - text_width) / 2 + 2, height - text_height * 1.6),
                caption.upper(),
                font=font,
                fill=255
            )
            draw.text(
                ((width - text_width) / 2 - 2, height - text_height * 1.6),
                caption.upper(),
                font=font,
                fill=255
            )
            draw.text(
                ((width - text_width) / 2, height - text_height * 1.6 + 2),
                caption.upper(),
                font=font,
                fill=255
            )
            draw.text(
                ((width - text_width) / 2, height - text_height * 1.6 - 2),
                caption.upper(),
                font=font,
                fill=255
            )
            draw.text(
                ((width - text_width) / 2, height - text_height * 1.6),
                caption.upper(),
                font=font
            )

        fp = StringIO()
        gf.writeGifToFile(fp, images, durations, 0, xys, disposes)
        filesize = fp.tell()
        if filesize > 2097152:
            return self.make_meme(original_image, caption, shrink * 0.8)

        fp.seek(0)
        image = b64encode(fp.read())
        payload = {
            'album_id': None,
            'image': image,
            'title': "Autogenerated Giphy Meme \"{0}\"".format(caption),
            'description': None
        }

        resp = requests.post(
            "https://api.imgur.com/3/image",
            data=payload,
            headers={
                "Authorization": "Client-ID c3bb952a0a07527"
            }
        )
        return resp.json()['data']['link']



from base64 import b64encode
from cStringIO import StringIO

import requests
from PIL import Image, ImageSequence, ImageDraw, ImageFont

from ..base import CommandBridle
from ...utils.images2gif import GifWriter


class FlashyGif(object):

    base_image = "https://dl.dropboxusercontent.com/u/176857/flash.gif"
    font_size = 50

    def __init__(self):
        response = requests.get(self.base_image)
        self.base = Image.open(StringIO(response.content))
        self.duration = self.base.info['duration'] / 1000.0
        self.font = ImageFont.truetype(
            'assets/fonts/impact.ttf',
            self.font_size
        )
        self.split_frames()

        self.default_palette = []
        for i in range(256):
            self.default_palette.extend((i, i, i))

    def resize(self, width, height):
        self.width = width
        self.height = height
        frames = []
        for im in self.frames:
            frames.append(im.resize((int(width), int(height))))

        self.frames = frames

    def split_frames(self):
        frames = []
        for frame in ImageSequence.Iterator(self.base):
            frames.append(frame.copy())
        gf = GifWriter()
        self.frames = gf.convertImagesToPIL(frames, False, 0)

    def resize_for_text(self, text):
        draw = ImageDraw.Draw(self.frames[0])
        self.text_width, self.text_height = draw.textsize(text, font=self.font)
        self.resize(self.text_width * 1.04, self.text_height * 1.1)

    def write(self, text):

        self.resize_for_text(text)

        self.durations = []
        self.xys = []
        self.disposes = []

        frames = []
        for im in self.frames:

            palette = self.default_palette[:]
            palette[0] = im.getpalette()[0]
            palette[1] = im.getpalette()[1]
            palette[2] = im.getpalette()[2]
            im.putpalette(palette)

            self.durations.append(self.duration)
            self.xys.append((0, 0))
            self.disposes.append(2)

            draw = ImageDraw.Draw(im)

            draw.text(
                (
                    self.text_width * 0.02,
                    -self.text_height * 0.05
                ),
                text,
                fill=255,
                font=self.font
            )
            frames.append(im)

        self.frames = frames

    def save(self):
        fp = StringIO()
        gf = GifWriter()
        gf.writeGifToFile(
            fp,
            self.frames,
            self.durations,
            0,
            self.xys,
            self.disposes
        )

        fp.seek(0)
        image = b64encode(fp.read())
        payload = {
            'album_id': None,
            'image': image,
            'title': "Autogenerated Flashy Text",
            'description': None
        }

        resp = requests.post(
            "https://api.imgur.com/3/image",
            data=payload,
            headers={
                "Authorization": "Client-ID c3bb952a0a07527"
            }
        )
        return resp.json()['data']['link']


class FlashyText(CommandBridle):

    class Meta(CommandBridle.Meta):
        command = 'flashy'
        description = 'Create a flashy message gif!'
        help_text = ['Usage: `/horse flashy <caption>`']

    def execute(self, user, channel, operands):
        if len(operands) == 0:
            return 'Can\'t flash with nothing, mate'

        flash = FlashyGif()
        flash.write(" ".join(operands).upper())
        image_url = flash.save()

        self.message(
            channel,
            image_url,
            username=user.real_name if user.real_name else user.name,
            icon_url=user.image
        )



import random

import pylast
import sqlobject

from ... import config
from ..base import CommandBridle
from ..base import BridleModel


class MusicFavourite(BridleModel):
    artist = sqlobject.StringCol()
    title = sqlobject.StringCol()
    fav_count = sqlobject.IntCol(default=1)


class LastFM(CommandBridle):

    class Meta(CommandBridle.Meta):
        command = "music"
        display_name = "Belfast Music"
        display_icon = "http://ryangrieve.com/labs/slack_icons/music.png"
        description = "Gets info on music playing in Unit12 via LastFM"
        help_text = [
            "Usage:",
            "\t`/horse music current` to get the currently playing track",
            "\t`/horse music recent` to get a list of recent tracks",
            "\t`/horse music fav` to add current track to favourites",
            "\t`/horse music suggest` to have horse suggest a song"
        ]

    tips = [
        "Type `/horse music fav` to favourite it!",
        "You can find out recently played songs with `/horse music recent`."
    ]

    def __init__(self, jockey):
        self.jockey = jockey
        self.network = pylast.LastFMNetwork(
            api_key=config.LASTFM_API_KEY,
            api_secret=config.LASTFM_API_SECRET,
            username=config.LASTFM_USERNAME,
            password_hash=pylast.md5(config.LASTFM_PASSWORD)
        )
        self.user_details = pylast.User("rehab_belfast", self.network)

    def current(self):
        msg = "Not playing anything. *Get on it!*"

        track = self.user_details.get_now_playing()
        text_tip = random.choice(self.tips)
        if track:
            msg = "Now playing: {0}. {1}".format(
                "*{0}* - _\"{1}\"_".format(
                    track.artist.name,
                    track.title
                ),
                text_tip
            )

        return msg

    def favourite(self):
        track = self.user_details.get_now_playing()
        if track:
            exists = MusicFavourite.selectBy(
                artist=track.artist.name,
                title=track.title
            ).getOne(None)
            if exists:
                exists.set(fav_count=exists.fav_count + 1)
                msg = "*{0}* - _\"{1}\"_ favourited {2} times!".format(
                    track.artist.name,
                    track.title,
                    exists.fav_count
                )
            else:
                MusicFavourite(
                    artist=track.artist.name,
                    title=track.title
                )
                msg = "*{0}* - _\"{1}\"_ added to favourite songs!".format(
                    track.artist.name,
                    track.title
                )
        else:
            msg = "There is no song currently playing."
        return msg

    def suggest(self):
        suggestions = list(MusicFavourite.select())
        if len(suggestions) > 0:
            suggestion = random.choice(suggestions)
            msg = "How about *{0}* - _\"{1}\"_?".format(
                suggestion.artist,
                suggestion.title
            )
        else:
            msg = "You haven't favourited any songs yet!"

        return msg

    def recent(self, count=5):
        recent_played = self.user_details.get_recent_tracks(count)
        tracks = [
            "\n\t*{0}* - _\"{1}\"_".format(
                t.track.artist.name,
                t.track.title
            ) for t in recent_played
        ]
        msg = 'The most recent tracks were:'
        for track in tracks:
            msg += track
        return msg.decode('utf-8')

    def execute(self, user, channel, operands):
        if len(operands) == 0 or operands[0] == 'current':
            message = self.current()
            self.message(channel, message)

        elif operands[0] == 'recent':
            count = 5
            if len(operands) > 1:
                try:
                    count = int(operands[1])
                except ValueError:
                    pass

            message = self.recent(count)
            self.message(channel, message)

        elif operands[0] == 'suggest':
            message = self.suggest()
            self.message(channel, message)

        elif operands[0] == 'fav':
            message = self.favourite()
            self.message(channel, message)



import sqlobject


class Base(sqlobject.SQLObject):
    created = sqlobject.DateTimeCol(default=sqlobject.DateTimeCol.now())


class User(Base):
    string_id = sqlobject.StringCol()
    username = sqlobject.StringCol()
    real_name = sqlobject.StringCol()
    image = sqlobject.StringCol()

    def __str__(self):
        return self.string_id


class Channel(Base):
    string_id = sqlobject.StringCol()
    name = sqlobject.StringCol()

    def __str__(self):
        return self.string_id
